const {classes: Cc, interfaces: Ci, utils: Cu} = Components,
  fPick = Cc['@mozilla.org/filepicker;1'].createInstance(Ci.nsIFilePicker),
  getWin = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator),
  base64Pref = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService).getBranch('extensions.base64encoder.'),
  STRINGS = Cc['@mozilla.org/intl/stringbundle;1'].getService(Ci.nsIStringBundleService).createBundle('chrome://srazzanoBase64Encoder/locale/encoder.properties');

var SoncoBase64Encoder = {
  Timer: null,
  init: function() { 
    window.removeEventListener('load', SoncoBase64Encoder.init, false);
    try {
      var tbBTN = document.getElementById('base64_Btn'), 
        val = base64Pref.getIntPref('viewIcon');
      tbBTN.style.listStyleImage = 'url(chrome://srazzanoBase64Encoder/skin/encoder' + val + '.png)';
      if (base64Pref.getBoolPref('hideToolbarIcon')) tbBTN.style.display = 'none';
      else tbBTN.style.display = '-moz-box';
    } catch(ex) {}
    try {
      var preImage = base64Pref.getBoolPref('previewImage');
      document.getElementById('base64_ViewCheck').checked = preImage;
      document.getElementById('base64_ViewIcon').style.opacity = '.2';
    } catch(ex) {}
    try {
      var htbi = base64Pref.getBoolPref('hideToolbarIcon');
      document.getElementById('hideToolbarIcon').checked = htbi;
      if (htbi) document.getElementById('viewIcon').disabled = htbi;
    } catch(ex) {}
    window.setTimeout(function() {SoncoBase64Encoder.setRefresh()}, 100);
  },

  setRefresh: function() {
    window.clearInterval(SoncoBase64Encoder.Timer);
    SoncoBase64Encoder.Timer = window.setInterval(function() {SoncoBase64Encoder.onPreview()}, 100);
  },

  destruct: function() {
    window.clearInterval(SoncoBase64Encoder.Timer);
  },

  openDialog: function() {
    window.openDialog('chrome://srazzanoBase64Encoder/content/encoder.xul', '_blank', 'chrome,dialog,resizable');
  },

  contextBtn: function(e) {
    if (!e.target.src) document.getElementById('base64_ContextBtn').style.display = 'none';
    else document.getElementById('base64_ContextBtn').style.display = '-moz-box';
  },

  getImage: function(aURL, nWidth, nHeight) {
    if (!aURL) return;
    else {
      setTimeout(function() {
        var xhr = new XMLHttpRequest(), 
          nsIMIMEService = Cc['@mozilla.org/mime;1'].getService(Ci.nsIMIMEService),
          mrw = getWin.getMostRecentWindow('base64:dialog');
        mrw.document.getElementById('base64_File').value = aURL;
        try {
          xhr.open('GET', aURL, true);
        } catch (aErr) {
          return;
        }
        xhr.responseType = 'arraybuffer';
        xhr.onload = function() {
          var mediaType, 
            dataURI, 
            binary, 
            bytes;
          bytes = new Uint8Array(xhr.response);
          binary = [].map.call(bytes, function(aByte) {return String.fromCharCode(aByte)}).join('');
          mediaType = xhr.getResponseHeader('Content-Type');
          var base64 = btoa(binary);
          if (mediaType.indexOf('image/') != 0) {
            try {
              mediaType = nsIMIMEService.getTypeFromURI(Services.io.newURI(aURL, null, null));
            } catch (aError) {
              mediaType = 'image/png';
              console.info(aError);
            }
          }
          dataURI = 'data:' + mediaType + ';base64,' + base64;
          mrw.document.getElementById('base64_Data').value = dataURI;
          var b64pd = STRINGS.GetStringFromName('b64pd'), 
            encIC = mrw.document.getElementById('base64_ViewCheck');
          encIC.label = b64pd + '\u2007' + nWidth + ' x ' + nHeight;
        }
        xhr.onerror = function() {
          return;
        };
        xhr.send(null);
      }, 20);
      setTimeout(function() {
        var mrw = getWin.getMostRecentWindow('base64:dialog'), 
          radio = base64Pref.getIntPref('viewSize');
        if (radio == 0) mrw.document.getElementById('radio_1').click();
        else mrw.document.getElementById('radio_2').click();
      }, 200);
    }
    SoncoBase64Encoder.openDialog();
  },

  onSelect: function() {
    var b64si = STRINGS.GetStringFromName('b64si');
    fPick.init(window, b64si, 0); 
    fPick.appendFilters(fPick.filterImages);
    if (fPick.show() != fPick.returnCancel) {
      document.getElementById('base64_File').value = 'file://' + fPick.fileURL.path.replace(/\\/g,'\/'); 
      var file = fPick.file,
        contentType = Cc['@mozilla.org/mime;1'].getService(Ci.nsIMIMEService).getTypeFromFile(file),
        fIS = Cc['@mozilla.org/network/file-input-stream;1'].createInstance(Ci.nsIFileInputStream),
        bIS = Cc['@mozilla.org/binaryinputstream;1'].createInstance(Ci.nsIBinaryInputStream);
      fIS.init(file, 0x01, 0600, 0);
      bIS.setInputStream(fIS);
      var base64 = btoa(bIS.readBytes(bIS.available()));
      bIS.close();
      fIS.close();
    } else return;
    document.getElementById('base64_Data').value = 'data:' + contentType + ';base64,' + base64;
    SoncoBase64Encoder.onViewSize();
  },

  onPaste: function() { 
    var encD = document.getElementById('base64_Data').value;
    window.opener.document.getElementById('urlbar').value = encD;
    window.opener.document.getElementById('urlbar').focus();
    SoncoBase64Encoder.onClose();
  },

  onCopy: function() { 
    var encD = document.getElementById('base64_Data').value;
    Cc['@mozilla.org/widget/clipboardhelper;1'].getService(Ci.nsIClipboardHelper).copyString(encD);
    SoncoBase64Encoder.onClose();
  }, 

  onSave: function() {
    var b64st = STRINGS.GetStringFromName('b64st'), 
      encF = document.getElementById('base64_File').value,
      encText = document.getElementById('base64_Data').value, 
      indexA = encF.lastIndexOf('/'), 
      indexB = encF.lastIndexOf('.'),
      fOS = Cc['@mozilla.org/network/file-output-stream;1'].createInstance(Ci.nsIFileOutputStream);
    fPick.appendFilters(fPick.filterText); 
    fPick.defaultExtension = '.txt'; 
    fPick.init(window, b64st, 1);
    fPick.defaultString = encF.substring(indexA + 1, indexB) + '.txt';
    if (fPick.show() != fPick.returnCancel) {
	 if (fPick.file.exists()) fPick.file.remove(true);
	 fPick.file.create(fPick.file.NORMAL_FILE_TYPE, 0666); 
	 fOS.init(fPick.file, 0x02, 0x200, null); 
	 fOS.write(encText, encText.length); 
	 fOS.close();
    } else return;
    SoncoBase64Encoder.onClose();
  },

  onImage: function() {
    if (!('Task' in window)) Cu.import('resource://gre/modules/Task.jsm');
    if (!('Downloads' in window)) Cu.import('resource://gre/modules/Downloads.jsm');
    var encI = document.getElementById('base64_View').src, 
      encF = document.getElementById('base64_File').value,
      indexA = encF.lastIndexOf('/'), 
      indexB = encF.lastIndexOf('.'), 
      imgExt;
    switch (encI.substring(0, encI.indexOf(';'))) {
      case 'data:image/svg+xml': imgExt = '.svg'; break;
      case 'data:image/x-icon': imgExt = '.ico'; break;
      case 'data:image/gif': imgExt = '.gif'; break;
      case 'data:image/jpeg': imgExt = '.jpeg'; break;
      case 'data:image/jpg': imgExt = '.jpg'; break;
      case 'data:image/bmp': imgExt = '.bmp'; break;
      case 'data:image/tiff': imgExt = '.tiff'; break;
      default: imgExt = '.png';
    }
    fPick.init(window, 'Save image', 1);
    fPick.appendFilters(fPick.filterImages);
    fPick.appendFilters(fPick.filterAll);
    fPick.defaultExtension = imgExt;
    fPick.defaultString = encF.substring(indexA + 1, indexB) + imgExt;
    if (fPick.show() != fPick.returnCancel) {
      var doDownload = 'fetch' in Downloads ? Downloads.fetch : Downloads.simpleDownload;
      Task.spawn(function() {
        doDownload.call(Downloads, encI, fPick.file.path);
      }).then(null, Cu.reportError);
    } else return;
    SoncoBase64Encoder.onClose();
  },

  onPreview: function() {
    var b64pd = STRINGS.GetStringFromName('b64pd'), 
      btnI = document.getElementById('btnImage'),
      encI = document.getElementById('base64_View'), 
      encIC = document.getElementById('base64_ViewCheck'),
      encF = document.getElementById('base64_File'), 
      encD = document.getElementById('base64_Data');
    try {
      encIC.checked = encIC.checked;
      if (!encIC.checked) {
        encI.setAttribute('src', '');
        encIC.label = b64pd;
        document.getElementById('base64_ViewIcon').style.opacity = '.2';
        base64Pref.setBoolPref('previewImage', false);
      } else if (encIC.checked && encD.value != '') {
        encI.setAttribute('src', encF.value);
        if (encF.value.match(/file:\/\//)) {
          btnI.disabled = true;
          encIC.label = b64pd + '\u2007' + encI.clientWidth + ' x ' + encI.clientHeight;
        } else btnI.disabled = false;
        document.getElementById('base64_ViewIcon').style.opacity = '1';
        base64Pref.setBoolPref('previewImage', true);
      }
    } catch(ex) {}
  },

  onViewSize: function() {
    var encD = document.getElementById('base64_Data'), 
      encS = document.getElementById('base64_Size'),
      encSL = document.getElementById('base64_SizeLabel'), 
      rg1 = document.getElementById('viewSize');
    if(rg1.value == 0 && encD.value != '') {
      var n = encD.textLength, 
        nS = new String(n), 
        i;
      if (n >= 1000) {
        i = nS.length;
        i -= 3; 
        encS.value = encSL.value + nS.substring(0, i) + '.' + nS.substring(i, nS.length - 1) + ' KB';
      } else encS.value = encSL.value + encD.textLength + ' Bytes';
    } else if (rg1.value == 1 && encD.value != '') encS.value = encSL.value + encD.textLength.toLocaleString() + ' Bytes';
    if (encD.textLength >= 1 ) encS.setAttribute('disabled', 'false');
    base64Pref.setIntPref('viewSize', rg1.value);
  },

  onSetIcon: function() {
    var mrw = getWin.getMostRecentWindow('navigator:browser'), 
      enable = base64Pref.getBoolPref('hideToolbarIcon');
    base64Pref.setBoolPref('hideToolbarIcon', !enable);
    document.getElementById('hideToolbarIcon').checked = enable;
    document.getElementById('viewIcon').disabled = base64Pref.getBoolPref('hideToolbarIcon');
    try {
      if (document.getElementById('hideToolbarIcon').checked) mrw.document.getElementById('base64_Btn').style.display = '-moz-box';
      else mrw.document.getElementById('base64_Btn').style.display = 'none';
    } catch(ex) {}
  },

  onViewIcon: function() {
    var mrw = getWin.getMostRecentWindow('navigator:browser'), 
      vI = document.getElementById('viewIcon').value;
    base64Pref.setIntPref('viewIcon', vI);
    try {
      mrw.document.getElementById('base64_Btn').style.listStyleImage = 'url(chrome://srazzanoBase64Encoder/skin/encoder' + vI + '.png)';
    } catch(ex) {}
  },

  onCloseWindow: function() {
    var bool = base64Pref.getBoolPref('closeWindow') != false ? false : true;
    base64Pref.setBoolPref('closeWindow', bool);
  },

  onClose: function() { 
    var encC = document.getElementById('base64_Close');
    if (encC.checked) close();
    else return;
  } 
}

window.addEventListener('load', SoncoBase64Encoder.init, false);
window.addEventListener('contextmenu', function(e) {SoncoBase64Encoder.contextBtn(e)});